@@ -10,21 +10,24 @@ try:
 except ImportError:  # pragma: no cover
     # this is for benefit of being able to build the docs on rtd.org
     class _Ldap3Module(object):
-        LDAPException = Exception
-        SEARCH_SCOPE_BASE_OBJECT = None
-        SEARCH_SCOPE_SINGLE_LEVEL = None
-        SEARCH_SCOPE_WHOLE_SUBTREE = None
-        STRATEGY_REUSABLE_THREADED = None
+        BASE = None
+        LEVEL = None
+        SUBTREE = None
+        REUSABLE = None
     ldap3 = _Ldap3Module()
+    LDAPException = Exception
+else:
+    LDAPException = ldap3.core.exceptions.LDAPException
 
 logger = logging.getLogger(__name__)
 
-
 _ord = ord if str is bytes else int
 
 _escape_for_search = {
     '*': '\\2A', '(': '\\28', ')': '\\29', '\\': '\\5C', '\0': '\\00'}
 
+__all__ = ['get_ldap_connector', 'get_groups', 'groupfinder']
+
 
 def escape_for_search(s):
     """Escape search string for LDAP according to RFC4515 when necessary."""
@@ -71,17 +74,18 @@ class _LDAPQuery(object):
 
         return self.cache.get(cache_key)
 
-    def execute(self, conn, **kw):
+    def execute(self, manager, **kw):
         cache_key = (self.base_dn % kw, self.filter_tmpl % kw)
 
         logger.debug('searching for %r', cache_key)
 
         result = self.query_cache(cache_key) if self.cache_period else None
         if result is None:
-            ret = conn.search(
-                search_scope=self.scope,
-                attributes=self.attributes, *cache_key)
-            result, ret = conn.get_response(ret)
+            with manager.connection() as conn:
+                ret = conn.search(
+                    search_scope=self.scope,
+                    attributes=self.attributes, *cache_key)
+                result, ret = conn.get_response(ret)
             if result is None:
                 result = []
             else:
@@ -109,10 +113,13 @@ class ConnectionManager(object):
     # noinspection PyShadowingNames
     def __init__(
             self, uri, bind=None, passwd=None, tls=None,
-            use_pool=True, pool_size=10, pool_lifetime=None, ldap3=ldap3):
+            use_pool=True, pool_size=10, pool_lifetime=3600,
+            get_info=None, ldap3=ldap3):
         self.ldap3 = ldap3
         uris = uri if isinstance(uri, (list, tuple)) else uri.split()
         self.uri = uri[0] if len(uris) == 1 else uris
+        if get_info is None:
+            get_info = ldap3.NONE
         servers = []
         for uri in uris:
             try:
@@ -126,18 +133,19 @@ class ConnectionManager(object):
             except ValueError:
                 host, port = host, 636 if use_ssl else 389
             server = self.ldap3.Server(
-                host, port=port, use_ssl=use_ssl, tls=tls)
+                host, port=port, use_ssl=use_ssl, tls=tls,
+                get_info=get_info)
             servers.append(server)
         self.server = servers[
             0] if len(servers) == 1 else self.ldap3.ServerPool(servers)
         self.bind, self.passwd = bind, passwd
         if use_pool:
-            self.strategy = ldap3.STRATEGY_REUSABLE_THREADED
+            self.strategy = ldap3.REUSABLE
             self.pool_name = 'pyramid_ldap3'
             self.pool_size = pool_size
             self.pool_lifetime = pool_lifetime
         else:
-            self.strategy = ldap3.STRATEGY_ASYNC_THREADED
+            self.strategy = ldap3.ASYNC
             self.pool_name = self.pool_size = self.pool_lifetime = None
 
     def __str__(self):
@@ -148,7 +156,7 @@ class ConnectionManager(object):
         if user:
             conn = self.ldap3.Connection(
                 self.server, user=user, password=password,
-                client_strategy=ldap3.STRATEGY_SYNC,
+                client_strategy=ldap3.SYNC,
                 auto_bind=True, lazy=False, read_only=True)
         else:
             conn = self.ldap3.Connection(
@@ -181,7 +189,7 @@ class Connector(object):
         A zero length password will always be considered invalid since it
         results in a request for "unauthenticated authentication" which should
         not be used for LDAP based authentication. See `section 5.1.2 of
-        RFC-4513 <http://tools.ietf.org/html/rfc4513#section-5.1.2>`_ for a
+        RFC-4513 <https://tools.ietf.org/html/rfc4513#section-5.1.2>`_ for a
         description of this behavior.
 
         If :meth:`pyramid.config.Configurator.ldap_set_login_query` was not
@@ -197,8 +205,9 @@ class Connector(object):
             raise ConfigurationError(
                 'ldap_set_login_query was not called during setup')
 
-        with self.manager.connection() as conn:
-            result = search.execute(conn, login=login, password=password)
+        result = search.execute(
+            self.manager, login=escape_for_search(login),
+            password=escape_for_search(password))
 
         if not result or len(result) > 1:
             return None
@@ -207,7 +216,7 @@ class Connector(object):
 
         try:
             self.manager.connection(login_dn, password).unbind()
-        except ldap3.LDAPException:
+        except LDAPException:
             logger.debug(
                 'Exception in authenticate with login %r', login,
                 exc_info=True)
@@ -235,21 +244,21 @@ class Connector(object):
         if search is None:
             raise ConfigurationError(
                 'set_ldap_groups_query was not called during setup')
-        with self.manager.connection() as conn:
-            try:
-                result = search.execute(conn, userdn=escape_for_search(userdn))
-            except ldap3.LDAPException:
-                logger.debug(
-                    'Exception in user_groups with userdn %r', userdn,
-                    exc_info=True)
-                return None
+        try:
+            result = search.execute(
+                self.manager, userdn=escape_for_search(userdn))
+        except LDAPException:
+            logger.debug(
+                'Exception in user_groups with userdn %r', userdn,
+                exc_info=True)
+            return None
 
         return result
 
 
 def ldap_set_login_query(
         config, base_dn, filter_tmpl,
-        scope=ldap3.SEARCH_SCOPE_SINGLE_LEVEL, attributes=None,
+        scope=ldap3.LEVEL, attributes=None,
         cache_period=0):
     """Configurator method to set the LDAP login search.
 
@@ -257,7 +266,7 @@ def ldap_set_login_query(
     ``filter_tmpl`` is a string which can be used as an LDAP filter:
     it should contain the replacement value ``%(login)s``.
     ``scope`` is any valid LDAP scope value
-    (e.g. ``ldap3.SEARCH_SCOPE_SINGLE_LEVEL``).
+    (e.g. ``ldap3.LEVEL`` or ``ldap3.SUBTREE``).
     ``attributes`` is a list of attributes that shall be returned
     (can also be set to None or ``ldap3.ALL_ATTRIBUTES``).
     ``cache_period`` is the number of seconds to cache login search results;
@@ -268,7 +277,7 @@ def ldap_set_login_query(
         config.set_ldap_login_query(
             base_dn='CN=Users,DC=example,DC=com',
             filter_tmpl='(sAMAccountName=%(login)s)',
-            scope=ldap3.SEARCH_SCOPE_SINGLE_LEVEL)
+            scope=ldap3.LEVEL)
 
     The registered search must return one and only one value to be considered
     a valid login.
@@ -290,7 +299,7 @@ def ldap_set_login_query(
 
 def ldap_set_groups_query(
         config, base_dn, filter_tmpl,
-        scope=ldap3.SEARCH_SCOPE_WHOLE_SUBTREE, attributes=None,
+        scope=ldap3.SUBTREE, attributes=None,
         cache_period=0):
     """ Configurator method to set the LDAP groups search.
 
@@ -298,7 +307,7 @@ def ldap_set_groups_query(
     ``filter_tmpl`` is a string which can be used as an LDAP filter:
     it should contain the replacement value ``%(userdn)s``.
     ``scope`` is any valid LDAP scope value
-    (e.g. ``ldap3.SEARCH_SCOPE_SINGLE_LEVEL``).
+    (e.g. ``ldap3.LEVEL`` or ``ldap3.SUBTREE``).
     ``attributes`` is a list of attributes that shall be returned
     (can also be set to None or ``ldap3.ALL_ATTRIBUTES``).
     ``cache_period`` is the number of seconds to cache groups search results;
@@ -309,7 +318,7 @@ def ldap_set_groups_query(
         config.set_ldap_groups_query(
             base_dn='CN=Users,DC=example,DC=com',
             filter_tmpl='(&(objectCategory=group)(member=%(userdn)s))'
-            scope=ldap3.SEARCH_SCOPE_WHOLE_SUBTREE)
+            scope=ldap3.SUBTREE)
 
     """
 
@@ -329,7 +338,9 @@ def ldap_set_groups_query(
 
 def ldap_setup(
         config, uri,
-        bind=None, passwd=None, use_tls=False, use_pool=True, pool_size=10):
+        bind=None, passwd=None, use_tls=False,
+        use_pool=True, pool_size=10, pool_lifetime=3600,
+        get_info=None):
     """Configurator method to set up an LDAP connection pool.
 
     - **uri**: ldap server uri(s) **[mandatory]**
@@ -338,18 +349,25 @@ def ldap_setup(
     - **passwd**: default password that will be used to bind a connector.
       **default: None**
     - **use_tls**: activate TLS when connecting. **default: False**
-    - **use_pool**: activates the pool. If False, will recreate a connector
-       each time. **default: True**
-    - **pool_size**: pool size. **default: 10**
+    - **use_pool**: activates the connection pool. If False, will recreate a
+      connector each time. **default: True**
+    - **pool_size**: connection pool size. **default: 10**
+    - **pool_lifetime**: number of seconds before recreating a new connection
+      when using a connection pool.  **default: 3600**
+    - **get_info**: specifies if schema or server specific info shall be read
+      for proper formatting of attributes.  **default: None**
     """
 
     manager = ConnectionManager(
-        uri, bind, passwd, use_tls, use_pool, pool_size if use_pool else None)
+        uri, bind, passwd, use_tls,
+        use_pool, pool_size if use_pool else None,
+        pool_lifetime if use_pool else None, get_info)
 
     def get_connector(request):
         return Connector(request.registry, manager)
 
-    config.set_request_property(get_connector, 'ldap_connector', reify=True)
+    config.add_request_method(
+        get_connector, 'ldap_connector', property=True, reify=True)
 
     intr = config.introspectable(
         'pyramid_ldap3 setup',
@@ -367,7 +385,7 @@ def get_ldap_connector(request):
     """
     connector = getattr(request, 'ldap_connector', None)
     if connector is None:
-        if ldap3.LDAPException is Exception:  # pragma: no cover
+        if LDAPException is Exception:  # pragma: no cover
             raise ImportError(
                 'You must install ldap3 to use an LDAP connector.')
         raise ConfigurationError(
